/*
 * Copyright 2010-2015 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package kotlin.reflect.jvm.internal.impl.serialization.deserialization.descriptors

import kotlin.reflect.jvm.internal.impl.descriptors.SourceElement
import kotlin.reflect.jvm.internal.impl.descriptors.annotations.AnnotationWithTarget
import kotlin.reflect.jvm.internal.impl.descriptors.annotations.Annotations
import kotlin.reflect.jvm.internal.impl.descriptors.impl.AbstractLazyTypeParameterDescriptor
import kotlin.reflect.jvm.internal.impl.resolve.descriptorUtil.builtIns
import kotlin.reflect.jvm.internal.impl.serialization.ProtoBuf
import kotlin.reflect.jvm.internal.impl.serialization.deserialization.*
import kotlin.reflect.jvm.internal.impl.types.KtType
import java.util.*

class DeserializedTypeParameterDescriptor(
        c: DeserializationContext, private val proto: ProtoBuf.TypeParameter, index: Int) : AbstractLazyTypeParameterDescriptor(c.storageManager, c.containingDeclaration, c.nameResolver.getName(proto.name), Deserialization.variance(proto.variance), proto.reified, index, SourceElement.NO_SOURCE) {
    private val typeDeserializer: TypeDeserializer = c.typeDeserializer
    private val typeTable: TypeTable = c.typeTable

    private val annotations = DeserializedAnnotationsWithPossibleTargets(c.storageManager) {
        c.components.annotationAndConstantLoader
                .loadTypeParameterAnnotations(proto, c.nameResolver)
                .map { AnnotationWithTarget(it, null) }
    }

    override fun getAnnotations(): Annotations = annotations

    override fun resolveUpperBounds(): Set<KtType> {
        val upperBounds = proto.upperBounds(typeTable)
        if (upperBounds.isEmpty()) {
            return setOf(this.builtIns.getDefaultBound())
        }
        val result = LinkedHashSet<KtType>(upperBounds.size())
        for (upperBound in upperBounds) {
            result.add(typeDeserializer.type(upperBound, Annotations.EMPTY))
        }
        return result
    }
}
